use std::ffi::c_void;
use std::mem;

use crate::CreateInterfaceFn;
use vtables::VTable;
use vtables_derive::has_vtable;
use vtables_derive::virtual_index;
use vtables_derive::VTable;

#[repr(u8)]
pub enum LuaState {
    Client,
    Server,
    Menu,
}

#[has_vtable]
#[derive(VTable)]
pub struct ILuaShared {}

#[allow(dead_code)]
impl ILuaShared {
    #[virtual_index(6)]
    pub fn get_lua_interface(&mut self, state: LuaState) -> *mut c_void {}

    pub fn from_ptr(ptr: *mut c_void) -> &'static ILuaShared {
        unsafe { mem::transmute(ptr) }
    }

    pub fn from_factory(factory: CreateInterfaceFn) -> Result<&'static ILuaShared, i32> {
        let mut result = 0;
        let ptr = unsafe { factory(b"LUASHARED003\0".as_ptr() as _, &mut result) };
        if ptr.is_null() {
            Err(result)
        } else {
            Ok(ILuaShared::from_ptr(ptr))
        }
    }
}
